#!/env/python3
'''
Utilities
'''

import os
import sys
from robot.api import TestSuiteBuilder

import config
from testspec import TestSpec, DOC_TC_LEVEL
from testpurpose import TP

DOC_CLAUSE_LVL_1 = config.DOC_CLAUSE_LVL_1
DOC_CLAUSE_LVL_2 = config.DOC_CLAUSE_LVL_2

DOC_CLAUSE_LVL_3 = config.DOC_CLAUSE_LVL_3
DOC_CLAUSE_LVL_4 = config.DOC_CLAUSE_LVL_4
 
DRY_RUN = config.DRY_RUN
QUIET = config.QUIET
BASE_SPEC = config.BASE_SPEC

# class TestDoc():
#     def __init__(self, src, doc_fn, doc_main_tit, commit_id, sol, api):
#         self.src = src
#         self.doc_fn = doc_fn
#         self.doc_main_tit = doc_main_tit
#         self.commit_id = commit_id
#         self.sol = sol
#         self.api = api

class DocReference():

    def __init__(self, sec_num_list=[1]):
        self._lvl = len(sec_num_list)
        self._nums = [int(i) for i in sec_num_list]

    def lvl(self):
        '''Returns the level of depth of the reference'''
        return self._lvl

    def __str__(self):
        '''To string'''
        return ".".join([str(x) for x in self._nums])

    def next_ref(self):
        '''Calculates the next reference and returns it'''
        nums_cpy = self._nums.copy()
        nums_cpy[self._lvl - 1] = self._nums[self._lvl - 1] + 1
        return DocReference(nums_cpy)

    def sub_ref(self):
        '''Return the reference of the first sub-section'''
        nums_cpy = self._nums.copy()
        nums_cpy.append(1)
        return DocReference(nums_cpy)

    @staticmethod
    def from_str(string):
        '''Builds a reference from a string'''
        return DocReference(string.split("."))

def keyword_to_line(k):
    '''
    Takes a Robot Framework keyword object and returns
    a string with keyword and arguments on a single line
    '''
    return str(k) + " " + " ".join(k.args)

def keywords_to_text(kws):
    '''
    Takes a list of Robot Framework words and returns a multiline text
    '''
    return "\n".join(map(keyword_to_line, kws))

def gen_test(this_test, spec, sec_ref, url):
    '''
    Generate the Docx part for an individual test
    '''

    ntp = TP(this_test.doc)

    if DRY_RUN:
        return
    if ntp.tp_id is not None:
        spec.add_heading(str(this_test), DOC_TC_LEVEL, ntp.tp_id)
    else:
        print("WARNING: Missing TP ID for test {}".format(this_test) )
        new_ref = sec_ref.next_ref()
        spec.add_heading_from_ref(str(this_test), new_ref)

    ntp.add_to_spec(
        spec, 
        keywords_to_text(this_test.keywords), 
        url
    )

def generate_void_clause(sec_ref, title, spec, sol):
    '''
    Generates a void clause. Generates Heading
    and a paragraph with text 'Void.'.
    '''
    print("++ Generating void clause: {} {} {}".format(sol, sec_ref, title))
    spec.add_sub_heading_from_ref(str(title), sec_ref, config.DOC_SUITE_LEVEL)
    spec.add_void_paragraph()

def gen_api_suite(sec_ref, suite, spec, workspace, commit_id, sol):
    '''
    Generates all tests in a Robot test suite (i.e .robot file). Generates Heading
    and a sub clause (heading + test description) for each test in the test suite.
    '''
    print("++ Generating test suite: {} {} {}".format(sol, sec_ref, suite))
    spec.add_sub_heading_from_ref(str(suite), sec_ref, config.DOC_SUITE_LEVEL)
    for i in suite.tests:
        url = build_test_url(commit_id, sol, workspace, str(suite)+".robot")

        log_line = ["++++ TD: ", sol, workspace, suite, i]
        not QUIET and print(",".join([str(x) for x in log_line]))

        gen_test(i, spec, sec_ref,url)

def gen_doc(src, doc_fn, commit_id, sol):
    '''
    Converts a Robot test suite to a word document

    src            file or directory containing Robot tests

    doc_fn         the filename for the output

    doc_main_tit   top level title for the section in the doc
    '''

    not QUIET and print("### robot2doc version " + config.VERSION)
    not QUIET and print("Starting..")
    
    script_dir = os.path.dirname(os.path.realpath(__file__))
    cwd = os.getcwd()

    # Using the absolute path has caused issues. The quick fix is to
    # move temporarily to the source directory
    os.chdir(script_dir)
    try:
        spec = TestSpec(BASE_SPEC)
    finally:
        os.chdir(cwd)

    spec = TestSpec()

    print("Loading tests from: " + str(src))
    workspace = []
    try:
        if isinstance(src, list):
            for suites_folder in src:
                workspace.append(TestSuiteBuilder().build(suites_folder))
        else:
            workspace.append(TestSuiteBuilder().build(src))
    except:
        print("Please check that first argument is a folder of Robot files or a single file.")
        sys.exit(-1)

    tot_suites = sum([len(x.suites) for x in workspace])
    tot_tests = sum([len(x.tests) for x in workspace])

    not QUIET and print("Loaded "+ str(tot_suites) + " test suites.")
    not QUIET and print("Loaded "+ str(tot_tests) + " tests.")
    
    api_dir_num = DOC_CLAUSE_LVL_1
    sec = DOC_CLAUSE_LVL_2
    subsec = DOC_CLAUSE_LVL_3

    sec_ref = DocReference([api_dir_num, sec, subsec])

    for api_dir in workspace:

        spec.add_main_heading("{} \t {}".format(sec_ref, api_dir), config.ANNEX_TITLE_LEVEL)

        print("-----> ", sec_ref, api_dir)
        file_ref = sec_ref.sub_ref()
        for test_file in api_dir.suites:
            print("-------> ", sec_ref, api_dir, file_ref, test_file)
            gen_api_suite(file_ref, test_file, spec, api_dir, commit_id, sol)
            file_ref = file_ref.next_ref()
            
        if len(api_dir.suites) == 0:
            suite = str(api_dir)
            gen_api_suite(sec, suite, spec, api_dir, commit_id, sol)
        
        sec_ref = sec_ref.next_ref()
        
            
    not QUIET and print("Saving to: " + doc_fn)
    not DRY_RUN and spec.save(doc_fn)
    not QUIET and print("Finished.")

def build_test_url(commit, sol, api, robot_file):
    '''Builds the URL structure'''

    configured_prefix = config.GIT_URL_PREFIX

    if configured_prefix != "":
        return configured_prefix + commit+"/"+sol+"/"+api+"/"+robot_file

    return None
