#!/env/python

import os
import docx
from docx.shared import Pt
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT, WD_BREAK_TYPE

import config as cfg

DOC_CLAUSE_LVL_1 = cfg.DOC_CLAUSE_LVL_1
DOC_CLAUSE_LVL_2 = cfg.DOC_CLAUSE_LVL_2

ANNEX_TITLE_LEVEL = cfg.ANNEX_TITLE_LEVEL
DOC_SUITE_LEVEL = cfg.DOC_SUITE_LEVEL
DOC_TC_LEVEL = cfg.DOC_TC_LEVEL

DOC_TC_FONT_SIZE = cfg.DOC_TC_FONT_SIZE
DOC_TC_FONT_NAME = cfg.DOC_TC_FONT_NAME

TP_TITLE = 'Test Purpose'
TC_TITLE = 'Test Case'

TABLE_STYLE = 'TestTable'

def add_hyperlink(paragraph, url, text, color, underline):
    """
    A function that places a hyperlink within a paragraph object.

    :param paragraph: The paragraph we are adding the hyperlink to.
    :param url: A string containing the required url
    :param text: The text displayed for the url
    :return: The hyperlink object
    """

    # This gets access to the document.xml.rels file and gets a new relation id value
    part = paragraph.part
    r_id = part.relate_to(url, docx.opc.constants.RELATIONSHIP_TYPE.HYPERLINK, is_external=True)

    # Create the w:hyperlink tag and add needed values
    hyperlink = docx.oxml.shared.OxmlElement('w:hyperlink')
    hyperlink.set(docx.oxml.shared.qn('r:id'), r_id, )

    # Create a w:r element
    new_run = docx.oxml.shared.OxmlElement('w:r')

    # Create a new w:rPr element
    rpr = docx.oxml.shared.OxmlElement('w:rPr')

    # Add color if it is given
    if not color is None:
        col = docx.oxml.shared.OxmlElement('w:color')
        col.set(docx.oxml.shared.qn('w:val'), color)
        rpr.append(col)

    # Remove underlining if it is requested
    if not underline:
        und = docx.oxml.shared.OxmlElement('w:u')
        und.set(docx.oxml.shared.qn('w:val'), 'none')
        rpr.append(und)

    # Join all the xml elements together add add the required text to the w:r element
    new_run.append(rpr)
    new_run.text = text
    hyperlink.append(new_run)

    paragraph._p.append(hyperlink)

    return hyperlink

class TestSpec():

    def __init__(self, path=None):

        cwd = os.getcwd()
        script_dir = os.path.dirname(os.path.realpath(__file__))
        
        if not path is None:
            print("Opening doc: "+path)
        else:
            print("Using base document: " + os.path.join(script_dir, cfg.BASE_SPEC))

        os.chdir(script_dir)
        
        self.doc = docx.Document(path or cfg.BASE_SPEC)
        os.chdir(cwd)

    @staticmethod
    def cell_text_bold(cell):
        cell.paragraphs[0].runs[0].font.bold = True

    @staticmethod
    def cell_text_centered(cell):
        cell.paragraphs[0].alignment = WD_PARAGRAPH_ALIGNMENT.CENTER

    @staticmethod
    def cell_text_mono(cell):
        cell.paragraphs[0].runs[0].font.name = DOC_TC_FONT_NAME
        cell.paragraphs[0].runs[0].font.size = Pt(DOC_TC_FONT_SIZE)

    @staticmethod
    def mk_ref(lv1, lv2="", lv3="", lv4="", lv5=""):
        secs = [str(x) for x in [lv1, lv2, lv3, lv4, lv5] if x != ""]
        return ".".join(secs)

    @staticmethod
    def mk_heading(txt, sec, subsec1="", subsec2="", subsec3=""):
        return TestSpec.mk_ref(sec, subsec1, subsec2, subsec3) + "\t" + txt

    @staticmethod
    def mk_heading_from_ref(txt, sec_ref):
        return str(sec_ref) + "\t" + txt

    @staticmethod
    def mk_tp_hdr(table, doc):
        hdr_cells = table.rows[0].cells
        hdr_cells[0].merge(hdr_cells[1])
        hdr_cells[0].text = TP_TITLE
        hdr_cells[0].paragraphs[0].style = doc.styles["TAH"]
        TestSpec.cell_text_bold(hdr_cells[0])
        TestSpec.cell_text_centered(hdr_cells[0])

    @staticmethod
    def mk_tc_hdr(table, doc):
        hdr_cells = table.add_row().cells
        hdr_cells[0].merge(hdr_cells[1])
        hdr_cells[0].text = TC_TITLE
        hdr_cells[0].paragraphs[0].style = doc.styles["TAH"]
        TestSpec.cell_text_bold(hdr_cells[0])
        TestSpec.cell_text_centered(hdr_cells[0])

    def add_commit_url(self, url: str):
        '''
        Adds a note to the document containing the URL to the location of the
        file, according to the configured URL prefix configured.
        '''
        self.doc.add_paragraph("", style=self.doc.styles["Normal"])
        note_par = self.doc.add_paragraph("NOTE: Robot code can be found at ")
        note_par.style = self.doc.styles["NO"]
        add_hyperlink(note_par, url, url, None, True)

    def add_tp(self, fields, testbehaviour):
        table = self.doc.add_table(cols=2, rows=1)
        table.style = self.doc.styles[TABLE_STYLE]
        table.autofit = True
        TestSpec.mk_tp_hdr(table, self.doc)
        for tpf in fields:
            new_row = table.add_row()
            row_cells = new_row.cells
            row_cells[0].width = 2
            row_cells[0].paragraphs[0].style = self.doc.styles["TAL"]
            row_cells[0].paragraphs[0].text = tpf.key.strip()
            row_cells[0].paragraphs[0].runs[0].font.bold = True
            row_cells[1].paragraphs[0].style = self.doc.styles["TAL"]
            row_cells[1].paragraphs[0].text = tpf.value.strip()
        TestSpec.mk_tc_hdr(table, self.doc)
        row_cells = table.add_row().cells
        row_cells[0].merge(row_cells[1])
        row_cells[0].text = testbehaviour
        for par in row_cells[0].paragraphs:
            par.style = self.doc.styles["PL"]
        # TestSpec.cell_text_mono(row_cells[0])

    def add_heading(self, txt, lvl, sec="1", subsec1="", subsec2="", subsec3=""):
        self.doc.add_heading(TestSpec.mk_heading(txt, sec, subsec1, subsec2, subsec3), lvl)

    def add_heading_from_ref(self, txt, ref):
        self.doc.add_heading(TestSpec.mk_heading_from_ref(txt, ref), ref.lvl())

    def save(self, path):
        self.doc.save(path)

    def add_main_heading(self, ref, lvl=None):
        # TODO Calculate HEADING LEVEL FROM REFERENCE LVL
        self.doc.add_heading(str(ref), lvl or ref.lvl())

    def add_sub_heading(self, txt, sec, subsec1, subsec2, subsec3):
        '''
        Add a sub heading to the document
        @txt Title of the heading
        @sec Numer of the section of the heading
        '''
        tsh = TestSpec.mk_heading(txt, sec, subsec1, subsec2, subsec3)
        self.doc.add_heading(tsh, DOC_SUITE_LEVEL)

    def add_sub_heading_from_ref(self, txt, sec_ref, lvl=None):
        '''
        Add a sub heading to the document
        @txt Title of the heading
        @sec Numer of the section of the heading
        '''
        tsh = TestSpec.mk_heading_from_ref(txt, sec_ref)
        self.doc.add_heading(tsh, lvl or sec_ref.lvl())

    def add_void_paragraph(self):
        '''
        Add a paragraph with the text Void.
        '''
        self.doc.add_paragraph("Void.", style=self.doc.styles["Normal"])
